/* Copyright (c) 2008 Telefónica Investigación y Desarrollo SAU.

This file is part of LinuxMD-XOS. LinuxMD-XOS is part of XtreemOS, a Linux-based Grid Operating System, see <http://www.xtreemos.eu> for more details. The XtreemOS project has been developed with the financial support of the European Commission's IST program under contract #FP6-033576.

LinuxMD-XOS is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 2 of the License, or (at your option) any later version.

LinuxMD-XOS is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with LinuxMD-XOS. If not, see <http://www.gnu.org/licenses/>.
*/

/**
 ** @file creds_store.h
 ** @brief libcredstore interface
 **
 ** This file should be included by any application or library that use
 ** libcredstore API directly. It's not needed in applications that use
 ** xos_getcred.
 **/


#ifndef CREDS_STORE_H
#define CREDS_STORE_H

/** @brief the maximun credential size (in bytes)
 **
 ** in KRS implementation, 10,000 bytes is a limit
 ** per user, that includes also metadata and
 ** names (if you use KRS, ensure that this limit is below
 ** 9800 to make space metadata). ZKRS implementation
 ** compress the credential, but compress ratio is not
 ** predectible
 **/
#define XOS_CREDENTIAL_MAXSIZE 10000

/** @brief struct with function pointers to use credstore
 **
 ** This struct with function pointers allows applications to use
 ** different possible credstore implementations without change the code
 **/
struct creds_store_funcs {
   /** @brief store specified credential
    **
    ** This functions stores the credential that user passes. If other
    ** credential is already stored, the new credential will overwrite it.
    **
    ** @retval 0 success
    ** @retval -1 failure
    **/
  int (*store_cred) (char* data_in_pem_format);
   /** @brief retrieve current credential from credstore, if config_name match
    **
    ** This function returns credential from credstore if not empty and
    ** config_name match or config_name is NULL.
    **
    ** @param config_name If parameter is NULL, this function returns the
    ** credential stored in the credstore (e.g. in Key Retention Service). If
    ** config_name is not NULL, first checks if config_name is the current
    ** configuration name in the credstore; returns NULL if they are different.
    **
    ** @retval credential
    ** @retval NULL if failure
    **/
  char* (*retrieve_cred) (char *config_name);
  /** @brief set a timeout after thich the credential is purged
   **
   ** Sets a timeout (in seconds) after which the credential is purged from credstore. Zero value cancels the timeout.
   **
   ** @param timeout The timeout in seconds. Zero value cancels the timeout.
   ** @retval 0 success
   ** @retval -1 failure
   **/
  int (*set_cred_timeout) (int timeout);
   /** @brief delete credential from credstore if it's not empty
   **
   ** Delete credential from credstore. Operation returns success (0) also if
   ** credstore is empty.
   **
   ** @retval 0 success
   ** @retval -1 failure
   **/
  int (*delete_cred) ();
   /** @brief return configuration name of the credential
    **
    ** This function returns the current configuration name of the key stored
    ** in credstore. If credstore is emtpy or operation faults, returns NULL.
    **
    ** If a credential is stored without specify a configuration name, 
    ** configuration name is filled with the value "default".
    **
    ** @retval "configuration name"
    ** @retval NULL failure or credstore is empty
    **/
  char * (*get_current_cred_configname) ();
  /** @brief sets the current configuration name of the credstore
   **
   ** This function set the new configuration name of the key stored in
   ** credstore. This function must be invoked after store a new credential.
   **
   ** In future releases, this function will be removed and config_name
   ** parameter will be pass to store_cred function.
   **
   ** @param config_name the new configuration name
   ** @retval 0 success
   ** @retval -1 failure
   **/
  int (*set_current_cred_configname)(char *config_name);
  /** @brief inform if specified credential is stored in credstore 
   ** 
   ** This function is similar to retrieve_cred, but returns 1 if the
   ** credential to this config_name is available, 0 otherwise, instead of
   ** the credential/NULL.
   **
   ** @param config_name If parameter is NULL, this function returns 1
   ** if credstore is not empty. If config_name is not NULL, also checks if
   ** config_name is the current configuration name in the credstore; returns 0
   ** if they are different.
   **
   ** @retval 1 available
   ** @retval 0 not available or failure
   **/

  int (*check_cred_is_available) (char *config_name); 
  /** @brief uses a private session credstore instead of user credstore
   **
   ** By default, all functions operate over current user credstore. This
   ** function creates a new private credstore session and all successive new
   ** calls of the process and its childs will use this new credstore session.
   ** It's important to call exit_session at the end of the session (although
   ** current three implementations don't need it because session will be
   ** killed when the program ends).
   **
   ** In the krs/zkrs implementations, this function invokes the syscall to
   ** start a new session. In the uskeystore implementation, this functions
   ** starts a new daemon using a fork and defines an environment variable
   ** with the address of the socket.
   ** The function returns a environment value that the user must use with
   ** putenv, if launching other processes within the same session. KRS
   ** implementation returns nothing, but uskeystore needs to modify the
   ** environment because new session processes use XOS_CREDSTORE_SOCKET to
   ** localize the socket of the session credstore daemon and to read the
   ** cookie and file handler required to authenticate.
   **
   ** @retval 0 success
   ** @retval -1 failure
   **/
  char *(*join_new_session) ();
  /** @brief end session credstore
   **
   ** If a user program calls join_new_session, it must call this function
   ** to end the session. In the KRS implementation this function do nothing
   ** (keystore session ends when the program that started the session ends).
   ** In the uskeystore implementation this function kills the session
   ** credstore daemon, but this call is not needed in Linux because
   ** join_new_session calls prctl to establish that the credstore daemon
   ** receives a SIGTERM when the parent process dies.
   **
   ** @retval 0 success
   ** @retval -1 failure
   **/
  int (*exit_session) ();
  /** @brief change user credstore after change UID
   **
   ** If a program changes the UID, is must call this function to change to
   ** the new user credstore. In the krs and zkrs implementations, this
   ** function invokes the syscall to change the keystore. In the uskeystore
   ** implementation, it launches a new xos_credstored.
   **
   ** IMPORTANT: this function checks UID, not EUID!!!! But with krs/zkrs is
   ** required that programmer also change FSUID (file system UID) that
   ** initially has the same value that FSUID, because FSUID is used to
   ** assign the owner of the key and for qouta accounting. If program
   ** does not need to recover root privileges, best solution is to change UID
   ** and EUID. If program needs recover root privileges, best option is
   ** continue with EUID=0, and change UID with setreuid and FSUID with
   ** setfsuid.
   **
   ** @retval 0 success
   ** @retval -1 failure
   **/
  int (*join_default_user_session) ();
};

typedef const struct creds_store_funcs * xoscredstore_h;
/** @brief get function pointers of available credstore default implementation 
 **
 ** If kernel supports Key Retention Service, default implementation is zkrs (
 ** KRS with key compress). If KRS is not available, default implementation is
 ** uskeystore.
 **
 ** @retval "function pointers" of credstore implementation 
 ** @retval NULL if failure
 **/
const struct creds_store_funcs * get_creds_store_funcs();
/** @brief get function pointers of requested credstore implementation 
 **
 ** Currently, there are three implementations available:
 ** <ul>
 ** <li>krs: use kernel Key Retention Service.</li>
 ** <li>zkrs: as krs, but compress/descompress credentials automatically.</li>
 ** <li>uskeystore: a user space implementation, based in a daemon for credstore.</li>
 ** </ul>
 ** @retval "function pointers" of credstore implementation 
 ** @retval NULL if failure
 **/
const struct creds_store_funcs * get_creds_store_funcs_by_impl(char* impl);

#endif
